// MKINITCPIO + UKI

use crate::{
    config::DriveConfig,
    linux::{arch_chroot, install_file},
    print_status,
};

/// Setup initramfs
pub fn setup_mkinitcpio(conf: &DriveConfig) {
    #[cfg(target_arch = "x86_64")]
    {
        print_status("Writing /etc/mkinitcpio.d/linux.preset");
        install_file(
            "/mnt/etc/mkinitcpio.d/linux.preset",
            include_str!("../root/mkinitcpio/linux.preset"),
            0o644,
        );
    }

    // Set kernel cmdline
    std::fs::create_dir_all("/mnt/etc/kernel").unwrap();

    if conf.encryption.is_some() {
        let block_uuid = find_uuid_by_dev(&conf.root.as_ref().unwrap()).unwrap();
        std::fs::write("/mnt/etc/kernel/cmdline",
        format!("rd.luks.options=timeout=30s,discard rd.luks.name={block_uuid}=root root=/dev/mapper/root rw")
    ).unwrap();
    } else {
        std::fs::write(
            "/mnt/etc/kernel/cmdline",
            format!("root={}", conf.root.as_ref().unwrap()),
        )
        .unwrap();
    }

    print_status("Writing /etc/mkinitcpio.conf");
    install_file(
        "/mnt/etc/mkinitcpio.conf",
        include_str!("../root/mkinitcpio.conf"),
        0o644,
    );

    arch_chroot(&["mkinitcpio", "--allpresets"], None, true);
}

fn find_uuid_by_dev(dev: &str) -> Option<String> {
    let dir_path = "/dev/disk/by-uuid";

    if let Ok(entries) = std::fs::read_dir(dir_path) {
        for entry in entries.flatten() {
            let path = entry.path();

            if let Ok(target) = std::fs::read_link(&path) {
                let resolved_path = path.parent().unwrap().join(&target);

                if resolved_path
                    .to_string_lossy()
                    .ends_with(dev.split("/").last().unwrap())
                {
                    // Match -> Return found Block Device UUID
                    if let Some(filename) = path.file_name() {
                        return Some(filename.to_string_lossy().to_string());
                    }
                }
            }
        }
    }

    None
}
