use std::time::Duration;

use crate::{
    config::OllamaConfig,
    linux::{arch_chroot, systemd_service_enable},
    pkg::install_pkgs,
    print_status,
};

/// Setup Ollama AI Service
pub fn setup_ollama(conf: &OllamaConfig) {
    if conf.gpu {
        install_pkgs(&["ollama-cuda"]);
    } else {
        install_pkgs(&["ollama"]);
    }

    systemd_service_enable("ollama.service");

    print_status("Pulling Models");

    let mut ollama_server = std::process::Command::new("arch-chroot")
        .arg("/mnt")
        .arg("/bin/runuser -u ollama -- env OLLAMA_MODELS=/var/lib/ollama HOME=/var/lib/ollama /usr/bin/ollama serve")
        .stdout(std::process::Stdio::piped())
        .spawn()
        .expect("Failed to start ollama server");

    std::thread::sleep(Duration::from_secs(5));

    let models = conf.models.clone().unwrap_or_default();

    for model in models {
        arch_chroot(&["ollama", "pull", &model], None, true);
    }

    ollama_server.kill().unwrap();
}
